#include <stdio.h>
#include <stdlib.h>
#include "hough.h"



int gen_table(int *table, int *len, int R_Petri, int width){
	float d1, d2;
	int radius1, radius2;
	int xr1, xr2, yr1, yr2;


	radius1 = R_Petri;
	radius2 = R_Petri+width;



	yr1 = 0;
	yr2 = 0;
	xr1 = radius1;
	xr2 = radius2;

	d1 = (float)(5/4)-radius1;
	d2 = (float)(5/4)-radius2;

	while (yr1 <= xr1){
		//inner circle
		if (d1 < 0){
			d1 = d1 + 2*yr1 + 3;
		} else {
			d1 = d1 + 2*(yr1-xr1) + 5;
			xr1--;
		}
		yr1++;

		//outer circle
		if (d2 < 0){
			d2 = d2 + 2*yr2 + 3;
		} else {
			d2 = d2 + 2*(yr2-xr2) + 5;
			xr2--;
		}
		yr2++;
		//Table update
		table[(yr1-1)*2]=xr1;
		table[(yr1-1)*2 + 1]=xr2-xr1;
	}

	*len=yr1;

	return 0;
}


int hough(int *x, int*y, char *img, int img_width, int img_height, int *table, int table_len, int mask_width, int mask_height, char treshold){

	long int *CorrMx;
	int corrx, corry;
	int x_offset, y_offset;
	int x_index, y_index;
	int i, j, k, l;
	long int max;
	

	corrx = img_width - mask_width;
	corry = img_height - mask_height;
	//debug
	//corrx = 1;
	//corry = 1;

	x_offset = mask_width/2;
	y_offset = mask_height/2;

	if(corrx<=0 || corry<=0) return 1;

	if ( ( CorrMx = (long int *)malloc(corrx * corry * sizeof(long int)) ) == NULL )  {
		printf("Can't alloc enought memory!\n" );
		return -1;
	}

	//initialize CorrMx
	for(i=0; i<corry; i++){
		for(j=0; j<corrx; j++){
			CorrMx[corrx*i + j]=0;
		}
	}

	for(i=0; i<corry; i++){
		for(j=0; j<corrx; j++){
			//debug
			//img[img_width*(i+y_offset)+(j+x_offset)]=255;
			
			for(k=0; k<table_len; k++){
				for(l=table[k*2]; l<(table[k*2] + table[k*2+1]); l++){
					//I.
					if(img[(x_offset + j + l) + ((y_offset + i - k)*img_width)]>treshold) CorrMx[corrx*i + j]++;
					//debug
					//img[(x_offset + j + l) + ((y_offset + i - k)*img_width)]=255;

					//II.
					if(img[(x_offset + j + l) + ((y_offset + i + k)*img_width)]>treshold) CorrMx[corrx*i + j]++;
					//debug
					//img[(x_offset + j + l) + ((y_offset + i + k)*img_width)]=255;

					//III.
					if(img[(x_offset + j - l) + ((y_offset + i + k)*img_width)]>treshold) CorrMx[corrx*i + j]++;
					//debug
					//img[(x_offset + j - l) + ((y_offset + i + k)*img_width)]=255;

					//IV.
					if(img[(x_offset + j - l) + ((y_offset + i - k)*img_width)]>treshold) CorrMx[corrx*i + j]++;
					//debug
					//img[(x_offset + j - l) + ((y_offset + i - k)*img_width)]=255;
				}
			}
			
		}
	}

	max = 0;
	//initialize CorrMx
	for(i=0; i<corry; i++){
		for(j=0; j<corrx; j++){
			if(CorrMx[corrx*i + j]>max){
				max = CorrMx[corrx*i + j];
				x_index = j;
				y_index = i;
			}
		}
	}

	*x = x_index + x_offset;
	*y = y_index + y_offset;

	free(CorrMx);
	return 0;
}