#include "uni1p_access.h"
#include <stdio.h>
#include <time.h>

#include <commdef.h>
#include "libUni1P.h"

#include "cv.h"
#include "highgui.h"
#include <ctype.h>

#include "hough.h"


#define READ_SEC 4
#define WRITE_SEC 8
#define SEC_SIZE 1024
#define IN_PX_BYTE_C 1
#define OUT_PX_BYTE_C 2

#define SIZEX 752 //
#define SIZEY 560 //

#define R_PETRI 240
#define PETRI_WIDTH 15
#define TRESHOLD 80

int Uni1pHpiWriteWord(  UNI1P_HANDLE hPlx,
                               tUni1pHpiModule dsp,
                               DWORD dest,
                               DWORD data)
{
  return Uni1pHpiBlock(hPlx, FALSE, dsp, dest, 1, &data);
}

DWORD Uni1pHpiReadWord(  UNI1P_HANDLE hPlx,
                                tUni1pHpiModule dsp,
                                DWORD src)
{
	DWORD buf[5] = { 0 };
  Uni1pHpiBlock(hPlx, TRUE, dsp, src, 1, buf);
  return buf[0];
}

/*
DWORD ReadDWord(P9054_HANDLE hPlx, DWORD addr)  {
    return P9054_ReadDWord(hPlx,
                    (addr & ADDR_SPACE_MASK) == CPLD_BASE_ADDR? CPLD_ADDR_SPACE : FPGA_ADDR_SPACE,
                    addr & ~ADDR_SPACE_MASK);
}


void WriteDWord(P9054_HANDLE hPlx, DWORD addr, DWORD data)  {
    P9054_WriteDWord(hPlx,
                    (addr & ADDR_SPACE_MASK) == CPLD_BASE_ADDR? CPLD_ADDR_SPACE : FPGA_ADDR_SPACE,
                    addr & ~ADDR_SPACE_MASK,
                    data);
}


*/

void usage(void){
	printf("Usage : <program_name> <dsp_binary_file> <image> [[<bitstream_0> <width_0>] ... [<bitstream_9> <width_9>]]\n");
}



int main (int argc, char *argv[])
{
	UNI1P_HANDLE hPlx;
	int i;
	int             j, k, block;
	int RetCode;
	unsigned long result=0;
	unsigned long buf = 0;
	unsigned long status=0;

	FILE            *f;
	FILE            *flist;
	char			imagepath[100];
	long            size;   // size buffer in bytes
	BYTE            *data;
	BYTE            *partdata;
		
	unsigned long	start_in_data;//4B
	unsigned long	start_out_data;//4B
	unsigned long *poleptr;//4B
		
    DWORD fpga = 0;
	DWORD stat;

	CvCapture* capture = 0;						// for video capture from webcam
	IplImage* source;							// pointer to image
	IplImage* grayimage;							// pointer to image
	IplImage* bigimage;							// pointer to image

	int x, y;
	int xc, yc;
	int x0 = 0;
	int y0 = 0;

	uchar	*imagerawdata, *raw_data_ptr;
	uchar *img_data_in=NULL, *data_in_ptr;//1B
	unsigned long *img_data_out;//4B
	uchar *img_data_out_n;//1B
	unsigned short *pptr;

	unsigned long out_data_len=0;
	unsigned long out_min=0;
	unsigned long out_max=0;

	clock_t start, stop;
	clock_t rstart, rstop;
	clock_t wait;

	int c;										// used for user input to capture "Esc" key
	int aloc;
	int esc;

	int *tabulka;
	int tab_len;

	int dx, dy;
	double d;
	double ColonyArea, TotArea;


	if(argc<3 || argc>22){
		usage();
		return 0;
	}

		// Initialize library
		Uni1pInit();
		//nastartovani komunikace s kartou
        hPlx = NULL;
        hPlx = Uni1pOpenBoard(0);

		if (hPlx)
        {
                RetCode=Uni1pHpiInit(hPlx);
                Sleep(400);
                fprintf( stderr, "Uni1pOpenBoard OK!\n");
        } else {
                fprintf( stderr, "Uni1pOpenBoard failed!\n");
        }



        if ( ( f = fopen( argv[1], "rb" ) ) == NULL )  {
            fprintf( stderr, "Can't open binary file\n");
            return -1;
        }
        // get file size
        fseek( f, 0, SEEK_END );
        size = ftell( f );
        fseek( f, 0, SEEK_SET );

        // alloc buffer
        if ( ( data = (BYTE*) malloc( size ) ) == NULL )  {
            fprintf( stderr, "Can't alloc enought memory!\n" );
            return -1;
        }
		// load file to memory
        if ( fread( data, 1, size, f ) != (size_t) size )  {
            fprintf( stderr, "Can't read file!\n" );
            return -1;
        }

		RetCode=Uni1pHpiDspReset(hPlx, UNI1P_DSP_M2);
		Sleep(200);
        if (Uni1pHpiBlock(hPlx, FALSE, UNI1P_DSP_M2, 0x0, (size)/4, (unsigned long *)data))
        {
                fprintf( stderr, "Uni1pHpiWriteBlock returned error!\n");
        }
        Sleep(200);
        Uni1pHpiDspRun(hPlx, UNI1P_DSP_M2);
        Sleep(200);


		fclose(f);


	/*********** Get capture from camera ***********/
/*
    capture = cvCaptureFromCAM( 0 );
    
    if(!capture)
    {
        fprintf(stderr,"Could not initialize camera...\n");
        return -1;
    } 
*/
	/*********** Create windows, autosize to images ***********/

    cvNamedWindow("Edge", CV_WINDOW_AUTOSIZE);
    cvNamedWindow("Origin", CV_WINDOW_AUTOSIZE);

	aloc=0;
	x = SIZEX;
	y = SIZEY;

	flist = fopen (argv[2] , "r");
	if (flist == NULL) perror ("Error opening file");
	else {
	 fscanf (flist, "%s", imagepath);
	 printf("Loading image %s\n",imagepath);
 	}

	for(;;){
		/*********** Capture image from camera ***********/
		start = clock();
/*
		source = cvQueryFrame(capture);
		if( !source )
			return 1;


		scale = source->width/SIZEX;
*/
		source = cvLoadImage(imagepath, CV_LOAD_IMAGE_COLOR);
		grayimage = cvCreateImage(cvSize(source->width, source->height), IPL_DEPTH_8U, 1);
		bigimage = cvCreateImage( cvSize( SIZEX, SIZEY), IPL_DEPTH_8U, 1 );
	 
		cvCvtColor( source, grayimage, CV_BGR2GRAY );
        //cvFlip(grayimage, NULL, 0);

		cvResize(grayimage, bigimage, CV_INTER_LINEAR );
		imagerawdata = bigimage->imageDataOrigin;

		/*********** Memory allocation ***********/
		if(aloc==0){

			// alloc image input data buffer
			if ( ( img_data_in = (uchar*) malloc( (SIZEX+2) * SIZEY * sizeof(uchar) ) ) == NULL )  {
				printf("Can't alloc enought memory!\n" );
				return -1;
			}

			// alloc image output data buffer
			if ( ( img_data_out = (unsigned long*) malloc( (SIZEX+2) * SIZEY * sizeof(unsigned long)/2 ) ) == NULL )  {
				printf("Can't alloc enought memory!\n" );
				return -1;
			}

			// alloc image normalized output data buffer
			if ( ( img_data_out_n = (uchar*) malloc( (SIZEX+2) * SIZEY * sizeof(uchar) ) ) == NULL )  {
				printf("Can't alloc enought memory!\n" );
				return -1;
			}

			aloc=1;
		}


		/*********** Find Petri dish  ***********/
		if ( ( tabulka = (int *)malloc(2 * (240 + 15) * sizeof(int)) ) == NULL )  {
			printf("Can't alloc enought memory!\n" );
			return -1;
		}

		gen_table(tabulka, &tab_len, R_PETRI, PETRI_WIDTH);
		hough(&xc, &yc, imagerawdata, SIZEX, SIZEY, tabulka, tab_len, ((R_PETRI+PETRI_WIDTH)*2), ((R_PETRI+PETRI_WIDTH)*2), TRESHOLD);
		//printf("Dish center: %d:%d\n", xc, yc);
		free(tabulka);
		cvCircle( source, cvPoint(cvRound(xc),cvRound(yc)), R_PETRI, CV_RGB(0,255,0), PETRI_WIDTH, 8, 0 );
		//set center of circle	
		if(x==512){
			y=512;
			x0=xc-512/2;
			y0=yc-512/2;
		}


		if((x0+x) > SIZEX) x0=0;
		if((y0+y) > SIZEY) y0=0;
		
		//PREPARE INPUT DATA
		data_in_ptr = img_data_in;
		raw_data_ptr = imagerawdata + y0*SIZEX + x0;
		ColonyArea=1;
		TotArea = 1;
		for(i=0; i<y; i++){
			for(j=0; j<x; j++){
				///add 0 at the begining of line
				if(j==0){
					*data_in_ptr = 0;
					*data_in_ptr++;
				}
	
				dx = (xc - (j+x0));
				dy = (yc - (i+y0));
				d = sqrt(dx*dx + dy*dy);

				if(*raw_data_ptr > TRESHOLD) {			
					if(d<(R_PETRI - PETRI_WIDTH)) {
						ColonyArea++;
						TotArea++;
					}

					*data_in_ptr=255;
				} else {

					if(d<(R_PETRI - PETRI_WIDTH)) {
						TotArea++;
					}

					*data_in_ptr=0;
				}

			
//				*data_in_ptr = *raw_data_ptr;

				*data_in_ptr++;
				*raw_data_ptr++;		

				///add 0 at the end of line
				if(j==x-1){
					*data_in_ptr = 0;
					*data_in_ptr++;
				}

			}
			raw_data_ptr += SIZEX - x;
		}

		poleptr = (unsigned long *)img_data_in;

		start_in_data = Uni1pHpiReadWord(hPlx, UNI1P_DSP_M2, SOBEL_INPUT);
		start_out_data = Uni1pHpiReadWord(hPlx, UNI1P_DSP_M2, SOBEL_OUTPUT);

		//start = clock();
		if (Uni1pHpiBlock(hPlx, FALSE, UNI1P_DSP_M2, start_in_data, ((x+2) * y)/(sizeof(unsigned long)), (unsigned long *)poleptr))
		{
				printf("Uni1pHpiWriteBlock returned error!\n");
		}
		//stop = clock();

		Uni1pHpiWriteWord(hPlx, UNI1P_DSP_M2, SOBEL_DATA_IN_LEN, ((x+2) * y));
		Uni1pHpiWriteWord(hPlx, UNI1P_DSP_M2, SOBEL_DATA_IN_READY, 1);
		


		start = clock();
		while( Uni1pHpiReadWord(hPlx, UNI1P_DSP_M2, SOBEL_DONE) != 1);
		stop = clock();
		printf("Time: %f s\n", ((double)(stop - start)/CLOCKS_PER_SEC));

		out_data_len = Uni1pHpiReadWord(hPlx, UNI1P_DSP_M2, SOBEL_DATA_OUT_LEN);

		//start = clock();
		if (Uni1pHpiBlock(hPlx, TRUE, UNI1P_DSP_M2, start_out_data, ((x+2) * y * 2)/(sizeof(unsigned long)), img_data_out ))
		{
				printf("Uni1pHpiReadBlock returned error!\n");
		}
		//stop = clock();

	
		pptr = (unsigned short *)img_data_out;

		for(i=0; i< ((x+2) * y); i++){

			//if((uchar)((sqrt(*pptr)))>TRESHOLD) img_data_out_n[i]=255;//(uchar)((sqrt(*pptr)));
			//else img_data_out_n[i]=0;

			img_data_out_n[i]=(uchar)((sqrt(*pptr)));

			//img_data_out_n[i]=(uchar)(((*pptr)));

			pptr++;
		}

		//cvSetImageData(bigimage, img_data_out_n, step);

		if((x0+x) > SIZEX) x0=0;
		if((y0+y) > SIZEY) y0=0;

		//PREPARE OUTPUT DATA
		//data_in_ptr = img_data_in;
		data_in_ptr = img_data_out_n;
		raw_data_ptr = imagerawdata + y0*SIZEX + x0;
		for(i=0; i<y; i++){
			for(j=0; j<x+2; j++){

				if(j==0 || j==(x+1)) {
					*data_in_ptr++;
				} else {
					
					dx = (xc - (j+x0));
					dy = (yc - (i+y0));
					d = sqrt(dx*dx + dy*dy);
					//printf("D: %f\n", d);
					
					if(i==0 || i==y-1 || j==1 || j==x) *raw_data_ptr =255;
					else if(d>=(R_PETRI - PETRI_WIDTH)) *raw_data_ptr = 0;
					else *raw_data_ptr = *data_in_ptr;
					
					
					//*raw_data_ptr = *data_in_ptr;

					*data_in_ptr++;
					*raw_data_ptr++;				
				}

			}
			raw_data_ptr += SIZEX - x;
		}

		


		//cvResize(bigimage, grayimage, CV_INTER_LINEAR );

		//cvCircle( source, cvPoint(xc,yc), 1, CV_RGB(0,0,255), 3 );

		cvShowImage( "Edge", bigimage);
		cvShowImage( "Origin", source);
		printf("Relative area of colonies: %f\n",ColonyArea/TotArea);

		wait = clock();
		
		while(((clock()-wait)/CLOCKS_PER_SEC)<0.5){
			cvShowImage( "Edge", bigimage);
			cvShowImage( "Origin", source);
		

			esc=0;
			/*********** Quit if user presses "Esc" ***********/
			c = cvWaitKey(10);
			//printf("KEY: %d\n", (char)c);
			if( (char)c == 27 ){
				cvSaveImage( "image.jpg", bigimage );
				esc=1;
				break;

			}


			if( (char)c == 87 || (char)c == 119 ){ //W
				y0-=10;
				if(y0<0) y0=0;
			}

			if( (char)c == 83 || (char)c == 115 ){ //S
				y0+=10;
				if(y0+y >=SIZEY) y0=SIZEY - y;
			}


			if( (char)c == 65 || (char)c == 97 ){ //A
				x0-=10;
				if(x0<0) x0=0;
			}

			if( (char)c == 68 || (char)c == 100 ){ //D
				x0+=10;
				if(x0+x >=SIZEX) x0=SIZEX - x;
			}


			if( (char)c == 78 || (char)c == 110 ){ //N
				if(!feof(flist)){
					if(feof(flist)) rewind(flist);
					fscanf (flist, "%s", imagepath);
					printf("Loading image %s\n",imagepath);
					if(feof(flist)) rewind(flist);
				} else {
					rewind(flist);
					fscanf (flist, "%s", imagepath);
					printf("Loading image %s\n",imagepath);
				}
				break;
			}


			if( (char)c == 43 ){ //+
				y+=10;			
				if(y>SIZEY) y=SIZEY;
			}

			if( (char)c == 45 ){ //-
				y-=10;			
				if(y<3) y=3;
			}

			//Uni1pWriteDWord( hPlx, SYSTEM_SMAP_STAT, fpga );
			
			if( (char)c >= 48 && (char)c <= (57-(10-((argc-3)/2))) && argc>3){//0..9
				if ( ( f = fopen( argv[(((char)c-48)*2)+3], "rb" ) ) == NULL )  {
					printf( "Can't open file: %s\n", argv[(((char)c-48)*2)+3] );
					return -1;
				}

				printf( "Loading bitstream: %s\n", argv[(((char)c-48)*2)+3] );
				printf("Image width: %d px\n", atoi(argv[(((char)c-48)*2)+3 + 1]));

				x=atoi(argv[(((char)c-48)*2)+3 + 1]);//1024;
				y=x*SIZEY/SIZEX;
				x0=0;
				y0=0;

				fpga = 0x10;//M2
				// get file size
				fseek( f, 0, SEEK_END );
				size = ftell( f );
				fseek( f, 0, SEEK_SET );

				// alloc buffer
				if ( ( partdata = (BYTE*) malloc( size ) ) == NULL )  {
				fprintf( stderr, "Can't alloc enought memory!\n" );
				return -1;
				}

				// load file to memory
				if ( fread( partdata, 1, size, f ) != (size_t) size )  {
				fprintf( stderr, "Can't read file!\n" );
				return -1;
				}

				rstart = clock();

				Uni1pWriteDWord( hPlx, SYSTEM_SMAP_STAT, fpga );
				k = 0;
				size = (size + 3) / 4;
				while ( size > 0 )  {
					stat = Uni1pReadDWord( hPlx, SYSTEM_SMAP_STAT );
					block = 15 - ((stat >> 16) & 0xf);
					if ( block > size ) block = size;

					for ( j = 0; j < block; j ++ )  {
						Uni1pWriteDWord( hPlx, SYSTEM_SMAP_DATA, ((DWORD *)partdata)[ k ++ ] );
					}
					size -= block;
				}
				
				Uni1pWriteDWord( hPlx, SYSTEM_SMAP_DATA, 0x20000000 ); //configuration interface  NOP to prevent Virtex II configuration "bug"

				fpga = 0;
				Uni1pWriteDWord( hPlx, SYSTEM_SMAP_STAT, fpga );

				rstop = clock();
				printf("Reconfiguration time : %ld ticks.\n", (rstop - rstart));
					
				fclose(f);
				printf("OK\n");
				break;
			}//bitstram select

		}//while ... delay
		cvReleaseImage(&source);
		cvReleaseImage(&grayimage);
		cvReleaseImage(&bigimage);

		stop = clock();
		//printf("Size: %ld B; Time: %f s; Rate: %f B/s\n", (x * y * 2)/(sizeof(unsigned long)),((double)(stop - start)/CLOCKS_PER_SEC), ((x * y * 2)/(sizeof(unsigned long)))/(((double)(stop - start)/CLOCKS_PER_SEC)));
		//printf("Time: %f s\n", ((double)(stop - start)/CLOCKS_PER_SEC));





//NEXT IMAGE

			printf("Relative area of colonies: %f\n",ColonyArea/TotArea);

			if(!feof(flist)){
				if(feof(flist)) rewind(flist);
				fscanf (flist, "%s", imagepath);
				printf("Loading image %s\n",imagepath);
				if(feof(flist)) rewind(flist);
			} else {
				rewind(flist);
				fscanf (flist, "%s", imagepath);
				printf("Loading image %s\n",imagepath);
			}


			if(esc==1) break;
    }



	free(data);
	free(img_data_out);
	free(img_data_out_n);
	free(img_data_in);
    cvReleaseCapture( &capture );
    cvDestroyAllWindows();

	fclose(flist);


}
//---------------------------------------------------------------------------






